/*
 * Terrier - Terabyte Retriever 
 * Webpage: http://terrier.org 
 * Contact: terrier{a.}dcs.gla.ac.uk
 * University of Glasgow - School of Computing Science
 * http://www.gla.ac.uk/
 * 
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Process.java.
 *
 * The Original Code is Copyright (C) 2004-2020 the University of Glasgow.
 * All Rights Reserved.
 *
 * Contributor(s):
 *   Craig Macdonald <craigm{a.}dcs.gla.ac.uk> (original author)
 *   
 */
package org.terrier.querying;
/** Process Interface. A Process implementation should be annotated with parameters
 *  of the Request object that need to be set for it to be successfully used
 *  using {@link ManagerRequisite}
 */
public interface Process {
	/** Run the instantiated process on the search request represented by q.
	  * @param manager The manager instance handling this search session.
	  * @param q the current query being processed
	  * 
	  */
	@Deprecated
	default void process(Manager manager, SearchRequest q) {
		this.process(manager, (Request) q);
	}
	
	default void process(Manager manager, Request q) {
		this.process(manager, (SearchRequest) q);
	}
	
	/**
	 * Returns the name of the processor.
	 * @return String the name of the processor.
	 */
	default String getInfo() {
		return this.getClass().getSimpleName();
	}
}
