/*
 * Terrier - Terabyte Retriever 
 * Webpage: http://terrier.org/
 * Contact: terrier{a.}dcs.gla.ac.uk
 * University of Glasgow - School of Computing Science
 * http://www.gla.ac.uk/
 * 
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is DocumentIndex.java
 *
 * The Original Code is Copyright (C) 2004-2020 the University of Glasgow.
 * All Rights Reserved.
 *
 * Contributor(s):
 *   Vassilis Plachouras <vassilis{a.}dcs.gla.ac.uk> (original contributor)
  *   Craig Macdonald <craigm{a.}dcs.gla.ac.uk>
  *   Erik Graf <graf{a.}dcs.gla.ac.uk>
 */
package org.terrier.structures;
import java.io.IOException;

/** 
 * Interface for a document index.
 */
public interface DocumentIndex 
{
    /** 
     * Return the document index entry based on its docid.
     * 
     * @return the document index entry based on its docid.
     */
    DocumentIndexEntry getDocumentEntry(int docid) throws IOException;
    
    /** 
     * Return the length of the document.
     * 
     * @return the length of the document.
     */
    int getDocumentLength(int docid) throws IOException;
    
    /** 
     * Return the number of documents.
     * 
     * @return the number of documents.
     */
    int getNumberOfDocuments();
}
