/*
 * Terrier - Terabyte Retriever 
 * Webpage: http://terrier.org/
 * Contact: terrier{a.}dcs.gla.ac.uk
 * University of Glasgow - School of Computing Science
 * http://www.gla.ac.uk/
 * 
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Pointer.java
 *
 * The Original Code is Copyright (C) 2004-2020 the University of Glasgow.
 * All Rights Reserved.
 *
 * Contributor(s):
 *   Craig Macdonald <craigm{a.}dcs.gla.ac.uk> (original contributor)
 */
package org.terrier.structures;

/** 
 * A base interface for all pointers.
 * A pointer is a "counter" of "things". 
 * @since 3.0
 */
public interface Pointer
{
    /** 
     * Return the number of "things" that this pointer refers to.
     * 
     * @return the number of "things" that this pointer refers to.
     */
    int getNumberOfEntries();
    
    /** 
     * Set the number of "things" that the pointer refers to.
     * 
     * @param n the number of "things" that the pointer refers to.
     *  */
    void setNumberOfEntries(int n);
    
    /** Returns a textual representation of the pointer alone */
    @Deprecated
    String pointerToString();

    /** 
     * Sets the pointer within this object to that represented by
     * the specified pointer.
     * 
     * @param p other pointer to update the pointer in this object.
     */
    void setPointer(Pointer p);
}
