/*
 * Terrier - Terabyte Retriever 
 * Webpage: http://terrier.org/
 * Contact: terrier{a.}dcs.gla.ac.uk
 * University of Glasgow - School of Computing Science
 * http://www.gla.ac.uk/
 * 
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is FieldPosting.java
 *
 * The Original Code is Copyright (C) 2004-2020 the University of Glasgow.
 * All Rights Reserved.
 *
 * Contributor(s):
 *   Craig Macdonald <craigm{a.}dcs.gla.ac.uk> (original contributor)
 */
package org.terrier.structures.postings;

/** 
 * Interface implemented by posting objects that have field information
 * recorded.
 * 
 * Field information can be obtained from the {@link #getFieldFrequencies()} method.
 *
 * @since 3.0
 * @author Craig Macdonald
 */
public interface FieldPosting extends Posting 
{
    /** 
     * Returns the frequencies of the term in each field of the document.
     * 
     * @return the frequencies of the term in each field of the document.
     *  */
    int[] getFieldFrequencies();
    
    /** 
     * Returns the lengths of the each field in the current document.
     * 
     * @return the lengths of the each field in the current document.
     *  */
    int[] getFieldLengths();
    
    @Deprecated
    void setFieldLengths(int[] newLengths);
}
