/*
 * Terrier - Terabyte Retriever 
 * Webpage: http://terrier.org/
 * Contact: terrier{a.}dcs.gla.ac.uk
 * University of Glasgow - School of Computing Science
 * http://www.gla.ac.uk/
 * 
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is WritablePosting.java
 *
 * The Original Code is Copyright (C) 2004-2020 the University of Glasgow.
 * All Rights Reserved.
 *
 * Contributor(s):
 *   Craig Macdonald <craigm{a.}dcs.gla.ac.uk> (original contributor)
 */
package org.terrier.structures.postings;

import java.io.Serializable;

import org.apache.hadoop.io.Writable;

/** 
 * Allow a {@link Posting} to be serialized to a DataOut.
 * 
 * @since 3.0
 * @author Craig Macdonald
 */
public interface WritablePosting extends Posting, Writable, Serializable 
{
    /**
     * Set the length of the document of the current posting in tokens. 
     * @param l length of the document of the current posting in tokens.
     */
    public void setDocumentLength(int l);

    /** It may be of benefit to update the frequency of the id in some applications.
     * This should not be used lightly, as it may break the decompression of other
     * postings in iterators. This was moved from Posting.
     */
    public void setId(int id);
}
