/*
 * Terrier - Terabyte Retriever 
 * Webpage: http://terrier.org 
 * Contact: terrier{a.}dcs.gla.ac.uk
 * University of Glasgow - School of Computing Science
 * http://www.gla.ac.uk/
 * 
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is TestEnglishTokeniser.java.
 *
 * The Original Code is Copyright (C) 2004-2020 the University of Glasgow.
 * All Rights Reserved.
 *
 * Contributor(s):
 *   Craig Macdonald <craigm{a.}dcs.gla.ac.uk> (original author)
 *   
 */

package org.terrier.indexing.tokenisation;



import static org.junit.Assert.assertFalse;

import java.io.IOException;

import org.junit.Test;

public class TestEnglishTokeniser extends BaseTestTokeniser {

	
	public TestEnglishTokeniser()
	{
		super.tokeniser = new EnglishTokeniser();
	}
	
	@Test public void testSingleTerms() throws Exception
	{
		testTokenisation(tokenise("a"), "a");
		testTokenisation(tokenise("hello"), "hello");
	}
	
	@Test public void testNull() throws Exception
	{
		TokenStream t = tokeniser.tokenise(null);
		assertFalse(t.hasNext());
	}
	
	
	
	//TODO: test length
	
	@Test public void testCheckTerms() throws Exception
	{
		testTokenisation(tokenise("a bb c"), "a", "bb", "c");
		testTokenisation(tokenise("a bbb c"), "a", "bbb", "c");
		testTokenisation(tokenise("a bbbb c"), "a", "c");
		
		testTokenisation(tokenise("a 0a00 c"), "a", "0a00", "c");
		testTokenisation(tokenise("a 0a000 c"), "a", "0a000", "c");
		testTokenisation(tokenise("a 0a0000 c"), "a", "c");
	}
	
	
	@Test public void testSingleLatinTerms() throws Exception
	{
		testTokenisation(tokenise("a\u0133a"), "a", "a");
	}
		
	@Test public void testMultipleTermsSpace() throws Exception
	{
		testTokenisation(tokenise("hello there"), "hello", "there");
		testTokenisation(tokenise("hello there     "), "hello", "there");
		testTokenisation(tokenise("a very     big hello there"), "a" , "very", "big", "hello", "there");
	}
	
	@Test public void testMulipleLongTerms() throws IOException
	{
		testTokenisation(tokenise("hello there mr wolf thisisareallylongword aye"), "hello", "there", "mr", "wolf", "aye");
	}
	
	@Test public void testMultipleTermsOther() throws Exception
	{
		for (char c : new char[]{'.', ',', '-', ':', '\\', '/', '?', '<', '>'})
			testTokenisation(tokenise("a"+c+"b"), "a", "b");
		testTokenisation(tokenise("a.b  "), "a", "b");
		testTokenisation(tokenise("a;b  -"), "a", "b");
		testTokenisation(tokenise("a;b  -"), "a", "b");
		testTokenisation(tokenise("...   a;b ?"), "a", "b");
	}
	
}
